local dewdrop = DewdropLib:GetInstance('1.0')
local tablet = TabletLib:GetInstance('1.0')

FuBar_RareTrackerFu = FuBarPlugin:GetInstance("1.2"):new({
	name          = "FuBar - RareTrackerFu",
	description   = "Fubar display for RareTracker.",
	version       = "0.1.1",
	releaseDate   = 2006-09-06,
	aceCompatible = 103,
	fuCompatible	= 10000,
	author        = "Updated by Tiran",
	email         = "",
	website       = "",
	category      = "Info",
	db            = AceDatabase:new("FuBar_RareTrackerFuDB"),
	defaults      = {
	  		sortBySubzone = false,
  	},
	cmd           = AceChatCmd:new({}, {}),

	hasIcon = false,
	hasNoText = false,

	loc = {
		
		DISPLAY_ZONE = "RareTracker: %d",
		DISPLAY_SUBZONE = "RareTracker: %d (%d)",
 
		TOOLTIP_HINT = "Left-click to scan your surroundings.",

		TOOLTIP_NO_KNOWN_MOBS = "No rare mobs known in %s.",	
		TOOLTIP_MOBS_KNOWN_IN_ZONE = "%d rare mob(s) known in %s:",
		TOOLTIP_MOBS_KNOWN_IN_SUB_ZONE = "%d of them near or in %s:",
		TOOLTIP_MOBS_KNOWN_IN_SUB_ZONE_ONLY = "%s:",
		TOOLTIP_MOBS_KNOWN_IN_UNKNOWN = "No subzone/never seen:",
		TOOLTIP_SEEN_AT = "Seen at:",
		TOOLTIP_MOB = " - %s, %s",
		
		MENU_SCAN = "Scan for nearby mobs",
		MENU_AUTOTARGET = "Allow autotargeting",
		MENU_TOOLTIP_LOCATION = "Use default tooltip location",
		MENU_SORT_BY_SUBZONE = "Sort tooltip by subzone",
		
	},

})

function FuBar_RareTrackerFu:Initialize()
    self.GetOpt = function(var) local v=self.db:get(self.profilePath, var) return v end
    self.TogOpt = function(var) return self.db:toggle(self.profilePath, var) end
    UIParentLoadAddOn("TNE_RareTrackerCore")
    RareTracker:RegisterForNotification("FuBar_RareTrackerFu_OnRareFound")

end

function FuBar_RareTrackerFu:Enable()

    FuBar_RareTrackerFu:RegisterEvent("ZONE_CHANGED_NEW_AREA", "UpdateText")
    FuBar_RareTrackerFu:RegisterEvent("ZONE_CHANGED", "UpdateText")

end


function FuBar_RareTrackerFu:UpdateText()

    local mobs = RareTracker:GetZone(GetRealZoneText())
    local submobs = RareTracker:GetZone(GetRealZoneText(), GetSubZoneText())
    local inZone, inSubZone = TNEUtils.TableLength(mobs) or "0", TNEUtils.TableLength(submobs)
    if (inSubZone) then
      self:SetText(format(self.loc.DISPLAY_SUBZONE, inZone, inSubZone))
    else
      self:SetText(format(self.loc.DISPLAY_ZONE, inZone))
    end

 end

function FuBar_RareTrackerFu:UpdateTooltip()
	tablet:SetTitle("")
	
	if (self.GetOpt("sortBySubzone")) then		
		self:AddSortedZoneToTooltip()
	else
		self:AddZoneToTooltip()
	end
	
end

function FuBar_RareTrackerFu:AddZoneToTooltip()
	local zone = GetRealZoneText()
	local subZone = nil
	local mobs = RareTracker:GetZone(zone, subZone)
	local num = TNEUtils.TableLength(mobs)
	local cat = tablet:AddCategory("columns",2,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 1,
			'child_text2R', 1,
			'child_text2G', 1,
			'child_text2B', 1)	
	
	if (num == 0) then
		cat:AddLine("text",  format(self.loc.TOOLTIP_NO_KNOWN_MOBS, zone))
	else
		cat:AddLine("text", format(self.loc.TOOLTIP_MOBS_KNOWN_IN_ZONE,num, zone), "text2", self.loc.TOOLTIP_SEEN_AT)		

		for key, data in mobs do
			local name, stats, coords = unpack(data)
			cat:AddLine("text", format(self.loc.TOOLTIP_MOB, FuBarUtils.Yellow(name), FuBarUtils.White(stats)),
				    "text2",coords)
		end
	end
	
	-- List the ones in this subzone
	subZone = GetSubZoneText()
	if (subZone) then
		mobs = RareTracker:GetZone(zone,subZone)
		num = TNEUtils.TableLength(mobs)
		if (num > 0) then
			cat:AddLine()
			cat:AddLine("text", format(self.loc.TOOLTIP_MOBS_KNOWN_IN_ZONE,num, subZone), "text2", self.loc.TOOLTIP_SEEN_AT)		
			for key, data in mobs do
				local name, stats, coords = unpack(data)
				cat:AddLine("text", format(self.loc.TOOLTIP_MOB, FuBarUtils.Yellow(name), FuBarUtils.White(stats)),
					    "text2",coords)
			end			
		end
		
	end
	
end

function FuBar_RareTrackerFu:AddSortedZoneToTooltip()
	local zone = GetRealZoneText()
	local num, mobs = RareTracker:GetSortedZoneData(zone)
	local cat = tablet:AddCategory("columns",2,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 1,
			'child_text2R', 1,
			'child_text2G', 1,
			'child_text2B', 1)	

	if (num == 0) then
		cat:AddLine("text",  format(self.loc.TOOLTIP_NO_KNOWN_MOBS, zone))
	else
		-- add all mobs in the zone
		for subZone, data in mobs do
			if (not (subZone == "")) then
				cat:AddLine("text", format(self.loc.TOOLTIP_MOBS_KNOWN_IN_SUB_ZONE_ONLY, subZone))
				for name, mob in data do
					local name, stats, coords = unpack(mob)
					cat:AddLine("text", format(self.loc.TOOLTIP_MOB, FuBarUtils.Yellow(name), FuBarUtils.White(stats)),
						    "text2",coords)
				end
			end
		end

		if (table.getn(mobs[""]) > 0) then
			cat:AddLine("text", self.loc.TOOLTIP_MOBS_KNOWN_IN_UNKNOWN)
			for name, mob in mobs[""] do
				local name, stats, coords = unpack(mob)
					cat:AddLine("text", format(self.loc.TOOLTIP_MOB, FuBarUtils.Yellow(name), FuBarUtils.White(stats)),
						    "text2",coords)
			end
		end	
	end
    
end

function FuBar_RareTrackerFu:ToggleCVar(var)
	SetCVar(var, GetCVar(var) == "1" and 0 or 1)
end


function FuBar_RareTrackerFu:OnClick()
	self:ToggleCVar("UnitNamePlayer")
end


function FuBar_RareTrackerFu:MenuSettings()

	
    if (level == 1) then
    	dewdrop:AddLine(
    		"text", self.loc.MENU_SCAN,
    		"func", function() RareTracker:Scan(); end,
    		"checked", nil
    	)
      
      	dewdrop:AddLine(
      		"text", self.loc.MENU_AUTOTARGET,
      		"func", function() TNE_RareTrackerCore_AutoTarget = not TNE_RareTrackerCore_AutoTarget; end,
      		"checked", TNE_RareTrackerCore_AutoTarget
      	)
      
      	dewdrop:AddLine(
		"text", self.loc.MENU_SORT_BY_SUBZONE,
		"func", function() self.TogOpt("sortBySubzone"); end,
		"checked", self.GetOpt("sortBySubzone")
      	)        
    end


end

function FuBar_RareTrackerFu:OnClick()
    RareTracker:Scan()
end

  
FuBar_RareTrackerFu:RegisterForLoad()

function FuBar_RareTrackerFu_OnRareFound()
	FuBar_RareTrackerFu:UpdateText()
end


